@@ -575,6 +575,17 @@ impl Expr {
         )
     }
 
+    /// Get the index values that would sort this expression.
+    pub fn arg_sort(self, reverse: bool) -> Self {
+        if has_expr(&self, |e| matches!(e, Expr::Wildcard)) {
+            panic!("wildcard not supported in unique expr");
+        }
+        self.map(
+            move |s: Series| Ok(s.argsort(reverse).into_series()),
+            Some(DataType::UInt32),
+        )
+    }
+
     /// Cast expression to another data type.
     pub fn cast(self, data_type: DataType) -> Self {
         Expr::Cast {
@@ -846,13 +857,18 @@ impl Expr {
 
     /// Check if the values of the left expression are in the lists of the right expr.
     #[allow(clippy::wrong_self_convention)]
-    pub fn is_in(self, list_expr: Expr) -> Self {
+    #[cfg(feature = "is_in")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "is_in")))]
+    pub fn is_in(self, other: Expr) -> Self {
         map_binary(
             self,
-            list_expr,
-            |left, list_ex| {
-                let list_array = list_ex.list()?;
-                left.is_in(list_array).map(|ca| ca.into_series())
+            other,
+            |left, other| {
+                left.is_in(&other).map(|ca| {
+                    let mut s = ca.into_series();
+                    s.rename(left.name());
+                    s
+                })
             },
             Some(Field::new("", DataType::Boolean)),
         )
@@ -1220,3 +1236,30 @@ impl Rem for Expr {
         binary_expr(self, Operator::Modulus, rhs)
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    use polars_core::df;
+    use polars_core::prelude::*;
+
+    #[test]
+    #[cfg(feature = "is_in")]
+    fn test_is_in() -> Result<()> {
+        let df = df![
+            "x" => [1, 2, 3],
+            "y" => ["a", "b", "c"]
+        ]?;
+        let s = Series::new("a", ["a", "b"]);
+
+        let out = df
+            .lazy()
+            .select([col("y").is_in(lit(s)).alias("isin")])
+            .collect()?;
+        assert_eq!(
+            Vec::from(out.column("isin")?.bool()?),
+            &[Some(true), Some(true), Some(false)]
+        );
+        Ok(())
+    }
+}
