@@ -33,6 +33,7 @@ pub struct LazyCsvReader<'a> {
     cache: bool,
     schema: Option<SchemaRef>,
     schema_overwrite: Option<&'a Schema>,
+    low_memory: bool,
 }
 
 impl<'a> LazyCsvReader<'a> {
@@ -47,6 +48,7 @@ impl<'a> LazyCsvReader<'a> {
             cache: true,
             schema: None,
             schema_overwrite: None,
+            low_memory: false,
         }
     }
 
@@ -100,6 +102,12 @@ impl<'a> LazyCsvReader<'a> {
         self
     }
 
+    /// Reduce memory usage in expensive of performance
+    pub fn low_memory(mut self, toggle: bool) -> Self {
+        self.low_memory = toggle;
+        self
+    }
+
     pub fn finish(self) -> LazyFrame {
         let mut lf: LazyFrame = LogicalPlanBuilder::scan_csv(
             self.path,
@@ -111,6 +119,7 @@ impl<'a> LazyCsvReader<'a> {
             self.cache,
             self.schema,
             self.schema_overwrite,
+            self.low_memory,
         )
         .build()
         .into();
@@ -151,7 +160,7 @@ impl IntoLazy for DataFrame {
 #[derive(Clone)]
 pub struct LazyFrame {
     pub(crate) logical_plan: LogicalPlan,
-    opt_state: OptState,
+    pub(crate) opt_state: OptState,
 }
 
 impl Default for LazyFrame {
@@ -332,11 +341,31 @@ impl LazyFrame {
         let opt_state = self.get_opt_state();
         let lp = self
             .get_plan_builder()
-            .sort(by_column.into(), reverse)
+            .sort(vec![col(by_column)], vec![reverse])
             .build();
         Self::from_logical_plan(lp, opt_state)
     }
 
+    /// Add a sort operation to the logical plan.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// use polars_core::prelude::*;
+    /// use polars_lazy::prelude::*;
+    ///
+    /// /// Sort DataFrame by 'sepal.width' column
+    /// fn example(df: DataFrame) -> LazyFrame {
+    ///       df.lazy()
+    ///         .sort_by_exprs(vec![col("sepal.width")], vec![false])
+    /// }
+    /// ```
+    pub fn sort_by_exprs(self, by_exprs: Vec<Expr>, reverse: Vec<bool>) -> Self {
+        let opt_state = self.get_opt_state();
+        let lp = self.get_plan_builder().sort(by_exprs, reverse).build();
+        Self::from_logical_plan(lp, opt_state)
+    }
+
     /// Reverse the DataFrame
     ///
     /// # Example
@@ -537,7 +566,10 @@ impl LazyFrame {
         let mut lp_arena = Arena::with_capacity(128);
         let lp_top = self.optimize(&mut lp_arena, &mut expr_arena)?;
 
-        toggle_string_cache(use_string_cache);
+        // if string cache was already set, we skip this and global settings are respected
+        if use_string_cache {
+            toggle_string_cache(use_string_cache);
+        }
         let planner = DefaultPlanner::default();
         let mut physical_plan =
             planner.create_physical_plan(lp_top, &mut lp_arena, &mut expr_arena)?;
@@ -1381,6 +1413,7 @@ mod test {
             .agg(vec![col("day").head(Some(2))])
             .collect()
             .unwrap();
+        dbg!(&out);
         let s = out
             .select_at_idx(1)
             .unwrap()
@@ -1967,25 +2000,105 @@ mod test {
     }
 
     #[test]
-    fn test_lazy_groupby_filter() {
+    fn test_lazy_groupby_filter() -> Result<()> {
         let df = df! {
             "a" => ["a", "a", "a", "b", "b", "c"],
             "b" => [1, 2, 3, 4, 5, 6]
-        }
-        .unwrap();
+        }?;
+
+        // We test if the filters work in the groupby context
+        // and that the aggregations can deal with empty sets
 
-        // test if it runs in groupby context
         let out = df
             .lazy()
             .groupby(vec![col("a")])
-            .agg(vec![col("b").filter(col("a").eq(lit("a"))).sum()])
+            .agg(vec![
+                col("b").filter(col("a").eq(lit("a"))).sum(),
+                col("b").filter(col("a").eq(lit("a"))).first(),
+                col("b").filter(col("a").eq(lit("e"))).mean(),
+                col("b").filter(col("a").eq(lit("a"))).last(),
+            ])
             .sort("a", false)
-            .collect()
-            .unwrap();
+            .collect()?;
 
         assert_eq!(
             Vec::from(out.column("b_sum").unwrap().i32().unwrap()),
-            [Some(6), Some(0), Some(0)]
+            [Some(6), None, None]
         );
+        assert_eq!(
+            Vec::from(out.column("b_first").unwrap().i32().unwrap()),
+            [Some(1), None, None]
+        );
+        assert_eq!(
+            Vec::from(out.column("b_mean").unwrap().f64().unwrap()),
+            [None, None, None]
+        );
+        assert_eq!(
+            Vec::from(out.column("b_last").unwrap().i32().unwrap()),
+            [Some(3), None, None]
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_groupby_projection_pd_same_column() -> Result<()> {
+        // this query failed when projection pushdown was enabled
+
+        let a = || {
+            let df = df![
+                "col1" => ["a", "ab", "abc"],
+                "col2" => [1, 2, 3]
+            ]
+            .unwrap();
+
+            df.lazy()
+                .select(vec![col("col1").alias("foo"), col("col2").alias("bar")])
+        };
+
+        let out = a()
+            .left_join(a(), col("foo"), col("foo"), None)
+            .select(vec![col("bar")])
+            .collect()?;
+
+        let a = out.column("bar")?.i32()?;
+        assert_eq!(Vec::from(a), &[Some(1), Some(2), Some(3)]);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_groupby_sort_slice() -> Result<()> {
+        let df = df![
+            "groups" => [1, 2, 2, 3, 3, 3],
+            "vals" => [1, 5, 6, 3, 9, 8]
+        ]?;
+        // get largest two values per groups
+
+        // expected:
+        // group      values
+        // 1          1
+        // 2          6, 5
+        // 3          9, 8
+
+        let out1 = df
+            .clone()
+            .lazy()
+            .sort("vals", true)
+            .groupby(vec![col("groups")])
+            .agg(vec![col("vals").head(Some(2)).alias("foo")])
+            .sort("groups", false)
+            .collect()?;
+
+        let out2 = df
+            .lazy()
+            .groupby(vec![col("groups")])
+            .agg(vec![col("vals").sort(true).head(Some(2)).alias("foo")])
+            .sort("groups", false)
+            .collect()?;
+
+        assert!(out1.column("foo")?.series_equal(out2.column("foo")?));
+        dbg!(out1, out2);
+        Ok(())
     }
 }
