@@ -41,6 +41,7 @@ pub enum ALogicalPlan {
         predicate: Option<Node>,
         aggregate: Vec<Node>,
         cache: bool,
+        low_memory: bool,
     },
     #[cfg(feature = "parquet")]
     ParquetScan {
@@ -70,8 +71,8 @@ pub enum ALogicalPlan {
     },
     Sort {
         input: Node,
-        by_column: String,
-        reverse: bool,
+        by_column: Vec<Node>,
+        reverse: Vec<bool>,
     },
     Explode {
         input: Node,
@@ -293,7 +294,7 @@ impl ALogicalPlan {
             } => Sort {
                 input: inputs[0],
                 by_column: by_column.clone(),
-                reverse: *reverse,
+                reverse: reverse.clone(),
             },
             Explode { columns, .. } => Explode {
                 input: inputs[0],
@@ -350,6 +351,7 @@ impl ALogicalPlan {
                 with_columns,
                 predicate,
                 cache,
+                low_memory,
                 ..
             } => {
                 let mut new_predicate = None;
@@ -368,6 +370,7 @@ impl ALogicalPlan {
                     predicate: new_predicate,
                     aggregate: exprs,
                     cache: *cache,
+                    low_memory: *low_memory,
                 }
             }
             DataFrameScan {
