@@ -158,6 +158,7 @@ pub enum LogicalPlan {
         /// Aggregations at the scan level
         aggregate: Vec<Expr>,
         cache: bool,
+        low_memory: bool,
     },
     #[cfg(feature = "parquet")]
     #[cfg_attr(docsrs, doc(cfg(feature = "parquet")))]
@@ -226,8 +227,8 @@ pub enum LogicalPlan {
     /// Sort the table
     Sort {
         input: Box<LogicalPlan>,
-        by_column: String,
-        reverse: bool,
+        by_column: Vec<Expr>,
+        reverse: Vec<bool>,
     },
     /// An explode operation
     Explode {
@@ -273,6 +274,7 @@ impl Default for LogicalPlan {
             predicate: None,
             aggregate: vec![],
             cache: true,
+            low_memory: false,
         }
     }
 }
@@ -370,7 +372,7 @@ impl fmt::Debug for LogicalPlan {
             }
             Sort {
                 input, by_column, ..
-            } => write!(f, "SORT {:?} BY COLUMN {}", input, by_column),
+            } => write!(f, "SORT {:?} BY {:?}", input, by_column),
             Explode { input, columns, .. } => {
                 write!(f, "EXPLODE COLUMN(S) {:?} OF {:?}", columns, input)
             }
@@ -526,7 +528,7 @@ impl LogicalPlan {
             Sort {
                 input, by_column, ..
             } => {
-                let current_node = format!("SORT by {} [{}]", by_column, id);
+                let current_node = format!("SORT BY {:?} [{}]", by_column, id);
                 self.write_dot(acc_str, prev_node, &current_node, id)?;
                 input.dot(acc_str, (branch, id + 1), &current_node)
             }
@@ -960,6 +962,7 @@ impl LogicalPlanBuilder {
         cache: bool,
         schema: Option<Arc<Schema>>,
         schema_overwrite: Option<&Schema>,
+        low_memory: bool,
     ) -> Self {
         let path = path.into();
         let mut file = std::fs::File::open(&path).expect("could not open file");
@@ -987,6 +990,7 @@ impl LogicalPlanBuilder {
             predicate: None,
             aggregate: vec![],
             cache,
+            low_memory,
         }
         .into()
     }
@@ -1126,7 +1130,7 @@ impl LogicalPlanBuilder {
         .into()
     }
 
-    pub fn sort(self, by_column: String, reverse: bool) -> Self {
+    pub fn sort(self, by_column: Vec<Expr>, reverse: Vec<bool>) -> Self {
         LogicalPlan::Sort {
             input: Box::new(self.0),
             by_column,
@@ -1389,48 +1393,48 @@ mod test {
             let df = lf.collect().unwrap();
             println!("{:?}", df);
         }
-
-        // check if optimization succeeds with selection of the left and the right (renamed)
-        // column due to the join
-        {
-            let lf = left
-                .clone()
-                .lazy()
-                .left_join(right.clone().lazy(), col("days"), col("days"), None)
-                .select(&[col("temp"), col("rain"), col("rain_right")]);
-
-            print_plans(&lf);
-            let df = lf.collect().unwrap();
-            println!("{:?}", df);
-        }
-
-        // check if optimization succeeds with selection of the left and the right (renamed)
-        // column due to the join and an extra alias
-        {
-            let lf = left
-                .clone()
-                .lazy()
-                .left_join(right.clone().lazy(), col("days"), col("days"), None)
-                .select(&[col("temp"), col("rain").alias("foo"), col("rain_right")]);
-
-            print_plans(&lf);
-            let df = lf.collect().unwrap();
-            println!("{:?}", df);
-        }
-
-        // check if optimization succeeds with selection of the left and the right (renamed)
-        // column due to the join and an extra alias
-        {
-            let lf = left
-                .lazy()
-                .left_join(right.lazy(), col("days"), col("days"), None)
-                .select(&[col("temp"), col("rain").alias("foo"), col("rain_right")])
-                .filter(col("foo").lt(lit(0.3)));
-
-            print_plans(&lf);
-            let df = lf.collect().unwrap();
-            println!("{:?}", df);
-        }
+        //
+        // // check if optimization succeeds with selection of the left and the right (renamed)
+        // // column due to the join
+        // {
+        //     let lf = left
+        //         .clone()
+        //         .lazy()
+        //         .left_join(right.clone().lazy(), col("days"), col("days"), None)
+        //         .select(&[col("temp"), col("rain"), col("rain_right")]);
+        //
+        //     print_plans(&lf);
+        //     let df = lf.collect().unwrap();
+        //     println!("{:?}", df);
+        // }
+        //
+        // // check if optimization succeeds with selection of the left and the right (renamed)
+        // // column due to the join and an extra alias
+        // {
+        //     let lf = left
+        //         .clone()
+        //         .lazy()
+        //         .left_join(right.clone().lazy(), col("days"), col("days"), None)
+        //         .select(&[col("temp"), col("rain").alias("foo"), col("rain_right")]);
+        //
+        //     print_plans(&lf);
+        //     let df = lf.collect().unwrap();
+        //     println!("{:?}", df);
+        // }
+        //
+        // // check if optimization succeeds with selection of the left and the right (renamed)
+        // // column due to the join and an extra alias
+        // {
+        //     let lf = left
+        //         .lazy()
+        //         .left_join(right.lazy(), col("days"), col("days"), None)
+        //         .select(&[col("temp"), col("rain").alias("foo"), col("rain_right")])
+        //         .filter(col("foo").lt(lit(0.3)));
+        //
+        //     print_plans(&lf);
+        //     let df = lf.collect().unwrap();
+        //     println!("{:?}", df);
+        // }
     }
 
     #[test]
