@@ -1,12 +1,11 @@
 use super::expressions as phys_expr;
 use crate::logical_plan::Context;
-use crate::physical_plan::executors::*;
 use crate::prelude::*;
 use crate::utils::{aexpr_to_root_names, aexpr_to_root_nodes, agg_source_paths, has_aexpr};
 use ahash::RandomState;
 use itertools::Itertools;
 use polars_core::prelude::*;
-use polars_core::{frame::groupby::GroupByMethod, utils::parallel_op};
+use polars_core::{frame::groupby::GroupByMethod, utils::parallel_op_series};
 use polars_io::ScanAggregation;
 use std::collections::HashSet;
 use std::sync::Arc;
@@ -73,7 +72,7 @@ impl PhysicalPlanner for DefaultPlanner {
 }
 
 impl DefaultPlanner {
-    fn create_physical_expressions(
+    pub fn create_physical_expressions(
         &self,
         exprs: Vec<Node>,
         context: Context,
@@ -128,12 +127,13 @@ impl DefaultPlanner {
                 predicate,
                 aggregate,
                 cache,
+                low_memory,
             } => {
                 let predicate = predicate
                     .map(|pred| self.create_physical_expr(pred, Context::Default, expr_arena))
                     .map_or(Ok(None), |v| v.map(Some))?;
                 let aggregate = aggregate_expr_to_scan_agg(aggregate, expr_arena);
-                Ok(Box::new(CsvExec::new(
+                Ok(Box::new(CsvExec {
                     path,
                     schema,
                     has_header,
@@ -145,7 +145,8 @@ impl DefaultPlanner {
                     predicate,
                     aggregate,
                     cache,
-                )))
+                    low_memory,
+                }))
             }
             #[cfg(feature = "parquet")]
             ParquetScan {
@@ -206,6 +207,8 @@ impl DefaultPlanner {
                 reverse,
             } => {
                 let input = self.create_initial_physical_plan(input, lp_arena, expr_arena)?;
+                let by_column =
+                    self.create_physical_expressions(by_column, Context::Default, expr_arena)?;
                 Ok(Box::new(SortExec {
                     input,
                     by_column,
@@ -257,8 +260,7 @@ impl DefaultPlanner {
                 // TODO: fix this brittle/ buggy state and implement partitioned groupby's in eager
                 let mut partitionable = true;
 
-                // currently only a single aggregation seems faster with ad-hoc partitioning.
-                if aggs.len() == 1 && keys.len() == 1 {
+                if keys.len() == 1 {
                     for agg in &aggs {
                         // make sure that we don't have a binary expr in the expr tree
                         let matches =
@@ -277,6 +279,7 @@ impl DefaultPlanner {
                             Expr::Agg(AggExpr::Min(_))
                             | Expr::Agg(AggExpr::Max(_))
                             | Expr::Agg(AggExpr::Sum(_))
+                            | Expr::Agg(AggExpr::Mean(_))
                             // first need to implement this correctly
                             // | Expr::Agg(AggExpr::Count(_))
                             | Expr::Agg(AggExpr::Last(_))
@@ -295,7 +298,7 @@ impl DefaultPlanner {
                 if apply.is_some() {
                     partitionable = false;
                 }
-                let phys_keys =
+                let mut phys_keys =
                     self.create_physical_expressions(keys, Context::Default, expr_arena)?;
                 if partitionable {
                     let phys_aggs = self.create_physical_expressions(
@@ -305,7 +308,7 @@ impl DefaultPlanner {
                     )?;
                     Ok(Box::new(PartitionGroupByExec::new(
                         input,
-                        phys_keys,
+                        phys_keys.pop().unwrap(),
                         phys_aggs,
                         aggs.into_iter()
                             .map(|n| node_to_exp(n, expr_arena))
@@ -399,9 +402,13 @@ impl DefaultPlanner {
                 let group_column =
                     self.create_physical_expr(partition_by, Context::Default, expr_arena)?;
                 let mut out_name = None;
-                let apply_column = aexpr_to_root_names(function, expr_arena)
-                    .pop()
-                    .expect("need a root column for a window function");
+                let mut apply_columns = aexpr_to_root_names(function, expr_arena);
+                if apply_columns.len() > 1 {
+                    return Err(PolarsError::ValueError(
+                        "Binary/Ternary function not yet supported in window expressions".into(),
+                    ));
+                }
+                let apply_column = apply_columns.pop().unwrap();
 
                 if let Alias(expr, name) = expr_arena.get(function) {
                     function = *expr;
@@ -506,7 +513,7 @@ impl DefaultPlanner {
                             }
                             Context::Default => {
                                 let function = NoEq::new(Arc::new(move |s: Series| {
-                                    parallel_op(|s| Ok(s.min_as_series()), s, None)
+                                    parallel_op_series(|s| Ok(s.min_as_series()), s, None)
                                 })
                                     as Arc<dyn SeriesUdf>);
                                 Ok(Arc::new(ApplyExpr {
@@ -526,7 +533,7 @@ impl DefaultPlanner {
                             }
                             Context::Default => {
                                 let function = NoEq::new(Arc::new(move |s: Series| {
-                                    parallel_op(|s| Ok(s.max_as_series()), s, None)
+                                    parallel_op_series(|s| Ok(s.max_as_series()), s, None)
                                 })
                                     as Arc<dyn SeriesUdf>);
                                 Ok(Arc::new(ApplyExpr {
@@ -546,7 +553,7 @@ impl DefaultPlanner {
                             }
                             Context::Default => {
                                 let function = NoEq::new(Arc::new(move |s: Series| {
-                                    parallel_op(|s| Ok(s.sum_as_series()), s, None)
+                                    parallel_op_series(|s| Ok(s.sum_as_series()), s, None)
                                 })
                                     as Arc<dyn SeriesUdf>);
                                 Ok(Arc::new(ApplyExpr {
@@ -605,7 +612,7 @@ impl DefaultPlanner {
                             Context::Default => {
                                 let function = NoEq::new(Arc::new(move |s: Series| {
                                     let len = s.len() as f64;
-                                    parallel_op(|s| Ok(s.sum_as_series()), s, None)
+                                    parallel_op_series(|s| Ok(s.sum_as_series()), s, None)
                                         .map(|s| s.cast::<Float64Type>().unwrap() / len)
                                 })
                                     as Arc<dyn SeriesUdf>);
